@@ -1,6 +1,5 @@
 jQuery(function () {
     var _ELEMENT_KEY_SEQ = 0;
-    var defaultColors = d3.scaleOrdinal(d3.schemeCategory10);
 
     function Lifecycle (name) {
         this.name = name;
@@ -10,7 +9,10 @@ jQuery(function () {
         this.defaults = {};
         this.transitions = [];
         this.decorations = {};
+        this.ticket_zoom = 'dynamic';
+        this.ticket_center = 'status';
 
+        this.defaultColor = '#547CCC';
         this._undoState = { undoStack: [], redoStack: [] };
         this._keyMap = {};
         this._statusMeta = {};
@@ -46,6 +48,14 @@ jQuery(function () {
             self.ticket_display = config.ticket_display;
         }
 
+        if (config.ticket_zoom) {
+            self.ticket_zoom = config.ticket_zoom;
+        }
+
+        if (config.ticket_center) {
+            self.ticket_center = config.ticket_center;
+        }
+
         jQuery.each(['initial', 'active', 'inactive'], function (i, type) {
             if (config[type]) {
                 self.statuses = self.statuses.concat(config[type]);
@@ -78,7 +88,7 @@ jQuery(function () {
             };
 
             if (!meta.color) {
-                meta.color = defaultColors(meta._key);
+                meta.color = self.defaultColor;
             };
         });
 
@@ -252,6 +262,8 @@ jQuery(function () {
             transitions: self.transitions,
 
             ticket_display: self.ticket_display,
+            ticket_zoom: self.ticket_zoom,
+            ticket_center: self.ticket_center,
             decorations: {},
             statusExtra: {},
             transitionExtra: {}
@@ -360,7 +372,7 @@ jQuery(function () {
     Lifecycle.prototype.deleteStatus = function (key) {
         var self = this;
 
-        self._saveUndoEntry();
+        self._saveUndoEntry(false);
 
         var statusName = self.statusNameForKey(key);
         if (!statusName) {
@@ -389,10 +401,12 @@ jQuery(function () {
             }
             return true;
         });
+
+        self._undoStateChanged();
     };
 
     Lifecycle.prototype.addTransition = function (fromStatus, toStatus) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var transition = {
             _key    : _ELEMENT_KEY_SEQ++,
@@ -407,6 +421,8 @@ jQuery(function () {
 
         transition.right = this.defaultRightForTransition(transition);
 
+        this._undoStateChanged();
+
         return transition;
     };
 
@@ -448,7 +464,7 @@ jQuery(function () {
     };
 
     Lifecycle.prototype.deleteTransition = function (key) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         this.transitions = jQuery.grep(this.transitions, function (transition) {
             if (transition._key == key) {
@@ -457,10 +473,12 @@ jQuery(function () {
             return true;
         });
         delete this._keyMap[key];
+
+        this._undoStateChanged();
     };
 
     Lifecycle.prototype.deleteDecoration = function (type, key) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         this.decorations[type] = jQuery.grep(this.decorations[type], function (decoration) {
             if (decoration._key == key) {
@@ -469,6 +487,8 @@ jQuery(function () {
             return true;
         });
         delete this._keyMap[key];
+
+        this._undoStateChanged();
     };
 
     Lifecycle.prototype.itemForKey = function (key) {
@@ -494,7 +514,7 @@ jQuery(function () {
     };
 
     Lifecycle.prototype.deleteActionForTransition = function (transition, key) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         transition.actions = jQuery.grep(transition.actions, function (action) {
             if (action._key == key) {
@@ -503,11 +523,13 @@ jQuery(function () {
             return true;
         });
         delete this._keyMap[key];
+
+        this._undoStateChanged();
     };
 
     Lifecycle.prototype.updateItem = function (item, field, newValue, skipUndo) {
         if (!skipUndo) {
-            this._saveUndoEntry();
+            this._saveUndoEntry(false);
         }
 
         var oldValue = item[field];
@@ -517,10 +539,14 @@ jQuery(function () {
         if (item._type == 'status' && field == 'name') {
             this.updateStatusName(oldValue, newValue);
         }
+
+        if (!skipUndo) {
+            this._undoStateChanged();
+        }
     };
 
     Lifecycle.prototype.createActionForTransition = function (transition) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var action = {
             _type : 'action',
@@ -528,15 +554,18 @@ jQuery(function () {
         };
         transition.actions.push(action);
         this._keyMap[action._key] = action;
+
+        this._undoStateChanged();
+
         return action;
     };
 
     Lifecycle.prototype.beginDragging = function () {
-        this._saveUndoEntry();
+        this._saveUndoEntry(true);
     };
 
     Lifecycle.prototype.beginChangingColor = function () {
-        this._saveUndoEntry();
+        this._saveUndoEntry(true);
     };
 
     Lifecycle.prototype.moveItem = function (item, x, y) {
@@ -544,6 +573,10 @@ jQuery(function () {
         item.y = y;
     };
 
+    Lifecycle.prototype.moveCircleRadiusPoint = function (circle, x, y) {
+        circle.r = Math.max(10, Math.sqrt(x**2 + y**2));
+    }
+
     Lifecycle.prototype.movePolygonPoint = function (polygon, index, x, y) {
         var point = polygon.points[index];
         point.x = x;
@@ -551,7 +584,7 @@ jQuery(function () {
     };
 
     Lifecycle.prototype.createStatus = function (x, y) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var name;
         var i = 0;
@@ -572,15 +605,18 @@ jQuery(function () {
             x:     x,
             y:     y
         };
-        item.color = defaultColors(item._key);
+        item.color = this.defaultColor;
 
         this._statusMeta[name] = item;
         this._keyMap[item._key] = item;
+
+        this._undoStateChanged();
+
         return item;
     };
 
     Lifecycle.prototype.createTextDecoration = function (x, y) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var item = {
             _key: _ELEMENT_KEY_SEQ++,
@@ -591,11 +627,13 @@ jQuery(function () {
         };
         this.decorations.text.push(item);
         this._keyMap[item._key] = item;
+
+        this._undoStateChanged();
         return item;
     };
 
     Lifecycle.prototype.createPolygonDecoration = function (x, y, type) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var item = {
             _key: _ELEMENT_KEY_SEQ++,
@@ -612,11 +650,13 @@ jQuery(function () {
         };
         this.decorations.polygon.push(item);
         this._keyMap[item._key] = item;
+
+        this._undoStateChanged();
         return item;
     };
 
     Lifecycle.prototype.createCircleDecoration = function (x, y, r) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var item = {
             _key: _ELEMENT_KEY_SEQ++,
@@ -633,11 +673,13 @@ jQuery(function () {
         };
         this.decorations.circle.push(item);
         this._keyMap[item._key] = item;
+
+        this._undoStateChanged();
         return item;
     };
 
     Lifecycle.prototype.createLineDecoration = function (x, y) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var item = {
             _key: _ELEMENT_KEY_SEQ++,
@@ -652,21 +694,25 @@ jQuery(function () {
         };
         this.decorations.line.push(item);
         this._keyMap[item._key] = item;
+
+        this._undoStateChanged();
         return item;
     };
 
     Lifecycle.prototype.update = function (field, value) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         if (field == 'on_create' || field == 'approved' || field == 'denied' || field == 'reminder_on_open' || field == 'reminder_on_resolve') {
             this.defaults[field] = value;
         }
-        else if (field == 'ticket_display') {
+        else if (field == 'ticket_display' || field == 'ticket_zoom' || field == 'ticket_center') {
             this[field] = value;
         }
         else {
             console.error("Unhandled field in Lifecycle.update: " + field);
         }
+
+        this._undoStateChanged();
     };
 
     Lifecycle.prototype._currentUndoFrame = function () {
@@ -688,22 +734,23 @@ jQuery(function () {
         return frame;
     };
 
-    Lifecycle.prototype._saveUndoEntry = function () {
-        var frame = this._currentUndoFrame();
-        this._undoState.undoStack.push(frame);
-        this._undoState.redoStack = [];
+    Lifecycle.prototype._undoStateChanged = function () {
+        this._canUndo = this._undoState.undoStack.length > 0;
+        this._canRedo = this._undoState.redoStack.length > 0;
 
         if (this.undoStateChangedCallback) {
             this.undoStateChangedCallback();
         }
     };
 
-    Lifecycle.prototype.hasUndoStack = function () {
-        return this._undoState.undoStack.length > 0;
-    };
+    Lifecycle.prototype._saveUndoEntry = function (notify) {
+        var frame = this._currentUndoFrame();
+        this._undoState.undoStack.push(frame);
+        this._undoState.redoStack = [];
 
-    Lifecycle.prototype.hasRedoStack = function () {
-        return this._undoState.redoStack.length > 0;
+        if (notify) {
+            this._undoStateChanged();
+        }
     };
 
     Lifecycle.prototype._rebuildKeyMap = function () {
@@ -749,9 +796,7 @@ jQuery(function () {
 
         this._restoreState(entry);
 
-        if (this.undoStateChangedCallback) {
-            this.undoStateChangedCallback();
-        }
+        this._undoStateChanged();
 
         return frame;
     };
@@ -769,15 +814,13 @@ jQuery(function () {
 
         this._restoreState(entry);
 
-        if (this.undoStateChangedCallback) {
-            this.undoStateChangedCallback();
-        }
+        this._undoStateChanged();
 
         return frame;
     };
 
     Lifecycle.prototype.cloneItem = function (source, x, y) {
-        this._saveUndoEntry();
+        this._saveUndoEntry(false);
 
         var clone = JSON.parse(JSON.stringify(source));
         clone._key = _ELEMENT_KEY_SEQ++;
@@ -792,9 +835,25 @@ jQuery(function () {
         }
 
         this._keyMap[clone._key] = clone;
+
+        this._undoStateChanged();
+
         return clone;
     };
 
+    Lifecycle.prototype.selectedRights = function () {
+        var rights = jQuery.map(this.transitions, function (transition) { return transition.right });
+
+        if (this.type == 'ticket') {
+            rights = rights.concat(['ModifyTicket', 'DeleteTicket']);
+        }
+        else if (this.type == 'asset') {
+            rights = rights.concat(['ModifyAsset']);
+        }
+
+        return jQuery.unique(rights.sort());
+    };
+
     RT.Lifecycle = Lifecycle;
 });
 
